@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2012-2014 by Tomasz Bojczuk                             *
+ *   Copyright (C) 2012-2015 by Tomasz Bojczuk                             *
  *   tomaszbojczuk@gmail.com                                               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -17,18 +17,37 @@
  ***************************************************************************/
 
 #include "texecutorsupply.h"
-#include "texam.h"
-#include "tglobals.h"
-#include "tlevel.h"
-#include <ttune.h>
+#include "tequalrand.h"
+#include <exam/texam.h>
+#include <exam/tlevel.h>
+#include <exam/tattempt.h>
+#include <music/ttune.h>
+#include <music/tmelody.h>
+#include <tglobals.h>
+#include <tscoreparams.h>
+#include <widgets/tintonationview.h>
 #include <mainwindow.h>
 #include <QMouseEvent>
 #include <QMessageBox>
 #include <QDateTime>
 #include <QDebug>
+#include <iostream>
 
 extern Tglobals *gl;
 
+void debugMelody(Tmelody* mel) {
+	for (int i = 0; i < mel->length(); ++i)
+		std::cout << mel->note(i)->p().toText().toStdString() << "\t";
+	std::cout << "\n";
+}
+
+void debugNotesStruct(QList<TnoteStruct>& st) {
+	Tmelody mel;
+	mel.fromNoteStruct(st);
+	debugMelody(&mel);
+}
+
+
 /*static*/
 
 bool TexecutorSupply::m_paramsMessage = false;
@@ -39,7 +58,7 @@ void TexecutorSupply::checkPlayCorrected(Tlevel* level) {
 	if (level->instrument == e_noInstrument) {
 		if (level->answerIsSound())
 			if (gl->instrument != e_noInstrument)
-				if (level->inScaleOf(gl->loString().getChromaticNrOfNote(), gl->hiNote().getChromaticNrOfNote()))
+				if (level->inScaleOf(gl->loString().chromatic(), gl->hiNote().chromatic()))
 					m_playCorrections = false;
 	} else
 			m_playCorrections = false;
@@ -79,6 +98,23 @@ void TexecutorSupply::checkGuitarParamsChanged(MainWindow* parent, Texam* exam)
 }
 
 
+QColor& TexecutorSupply::answerColor(const TQAunit* answer) {
+	return answerColor(answer->mistake());
+}
+
+
+QColor& TexecutorSupply::answerColor(quint32 mistake) {
+	if (mistake == (quint32)TQAunit::e_correct)
+    return gl->EanswerColor;
+  else if (!(mistake & TQAunit::e_wrongPos) && !(mistake & TQAunit::e_wrongNote) && !(mistake & TQAunit::e_veryPoor))
+    return gl->EnotBadColor;
+  else
+    return gl->EquestionColor;
+}
+
+
+/* end of static */
+
 TexecutorSupply::TexecutorSupply(Tlevel* level, QObject* parent) :
   QObject(parent),
   m_level(level),
@@ -92,18 +128,30 @@ TexecutorSupply::TexecutorSupply(Tlevel* level, QObject* parent) :
 	m_hiFret = m_level->hiFret;
   calcQAPossibleCount();
 	checkPlayCorrected(level);
+	if (m_level->useKeySign && !m_level->isSingleKey)
+		m_randKey = new TequalRand(m_level->hiKey.value() - m_level->loKey.value() + 1, m_level->loKey.value());
+	else
+		m_randKey = 0;
 }
 
+
+TexecutorSupply::~TexecutorSupply()
+{
+	if (m_randKey)
+		delete m_randKey;
+}
+
+
 //##########################################################################################
 //#######################     METHODS       ################################################
 //##########################################################################################
 
 
-void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
+void TexecutorSupply::createQuestionsList(QList<TQAgroup> &list) {
 	char openStr[6];
 //       for (int i = 0; i < 6; i++)
 	for (int i = 0; i < gl->Gtune()->stringNr(); i++)
-			openStr[i] = gl->Gtune()->str(i + 1).getChromaticNrOfNote();
+			openStr[i] = gl->Gtune()->str(i + 1).chromatic();
 		
 		/** FIXING MISTAKE RELATED WITH A NEW VALIDATION WAY DURING SAVING NEW LEVEL 
 			* When there is no guitar in a level,
@@ -113,12 +161,12 @@ void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
 		m_level->onlyLowPos = true;
 
 	if (!m_playCorrections || m_level->instrument != e_noInstrument || m_level->showStrNr || m_level->canBeGuitar()) {
-		qDebug() << "Question list created fret by fret. Tune:" << gl->Gtune()->name << gl->Gtune()->stringNr();
+// 		qDebug() << "Question list created fret by fret. Tune:" << gl->Gtune()->name << gl->Gtune()->stringNr();
 		if (m_level->instrument == e_noInstrument && gl->instrument != e_noInstrument) {
-// 			if (Tnote(gl->hiString().getChromaticNrOfNote() + m_hiFret).getChromaticNrOfNote() < m_level->hiNote.getChromaticNrOfNote())
-// 					m_hiFret = m_level->hiNote.getChromaticNrOfNote() - gl->hiString().getChromaticNrOfNote();
-// 			if (Tnote(gl->loString().getChromaticNrOfNote() + m_loFret).getChromaticNrOfNote() > m_level->loNote.getChromaticNrOfNote())
-// 				m_loFret = gl->loString().getChromaticNrOfNote() - m_level->loNote.getChromaticNrOfNote();
+// 			if (Tnote(gl->hiString().chromatic() + m_hiFret).chromatic() < m_level->hiNote.chromatic())
+// 					m_hiFret = m_level->hiNote.chromatic() - gl->hiString().chromatic();
+// 			if (Tnote(gl->loString().chromatic() + m_loFret).chromatic() > m_level->loNote.chromatic())
+// 				m_loFret = gl->loString().chromatic() - m_level->loNote.chromatic();
 			char hi = m_hiFret, lo = m_loFret;
 			if (!m_level->adjustFretsToScale(lo, hi))
 					qDebug() << "Cant adjust fret range. Corrections will be played!";
@@ -130,9 +178,9 @@ void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
 		for(int s = 0; s < gl->Gtune()->stringNr(); s++) {
 				if (m_level->usedStrings[gl->strOrder(s)])// check string by strOrder
 						for (int f = m_loFret; f <= m_hiFret; f++) {
-								Tnote n = Tnote(gl->Gtune()->str(gl->strOrder(s) + 1).getChromaticNrOfNote() + f);
-							if (n.getChromaticNrOfNote() >= m_level->loNote.getChromaticNrOfNote() &&
-										n.getChromaticNrOfNote() <= m_level->hiNote.getChromaticNrOfNote()) {
+								Tnote n = Tnote(gl->Gtune()->str(gl->strOrder(s) + 1).chromatic() + f);
+							if (n.chromatic() >= m_level->loNote.chromatic() &&
+										n.chromatic() <= m_level->hiNote.chromatic()) {
 								bool hope = true; // we still have hope that note is proper for the level
 								if (m_level->onlyLowPos) {
 									if (s > 0) {
@@ -148,7 +196,7 @@ void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
 								if (hope && m_level->useKeySign && m_level->onlyCurrKey)
 									hope = isNoteInKey(n);
 								if (hope) {
-										if (n.acidental && (!m_level->withFlats && !m_level->withSharps))
+										if (n.alter && (!m_level->withFlats && !m_level->withSharps))
 												continue;
 										else {
 											TfingerPos ff = TfingerPos(gl->strOrder(s) + 1, f);
@@ -159,14 +207,14 @@ void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
 				}
 		}
 	} else {
-		qDebug() << "Question list created note by note";
-		for (int nNr = m_level->loNote.getChromaticNrOfNote(); nNr <= m_level->hiNote.getChromaticNrOfNote(); nNr++) {
+// 		qDebug() << "Question list created note by note";
+		for (int nNr = m_level->loNote.chromatic(); nNr <= m_level->hiNote.chromatic(); nNr++) {
 			Tnote n = Tnote(nNr);
 			bool hope = true; // we still have hope that note is proper for the level
 			if (hope && m_level->useKeySign && m_level->onlyCurrKey)
 					hope = isNoteInKey(n);
 			if (hope) {
-				if (n.acidental && (!m_level->withFlats && !m_level->withSharps))
+				if (n.alter && (!m_level->withFlats && !m_level->withSharps))
 						continue;
 				else {
 						TfingerPos ff = TfingerPos();
@@ -195,41 +243,22 @@ void TexecutorSupply::createQuestionsList(QList<TQAunit::TQAgroup> &list) {
 //        qDebug() << m_fretFretList.at(i) << (int)list[m_fretFretList.at(i)].pos.str() << "f"
 //                << (int)list[m_fretFretList.at(i)].pos.fret() << " note: "
 //                << QString::fromStdString(list[m_fretFretList.at(i)].note.getName());
-      qDebug() << "question list for only guitar created\nnumber:" << m_fretFretList.size() <<
-          " among:" << list.size();
+//       qDebug() << "question list for only guitar created\nnumber:" << m_fretFretList.size() <<
+//           " among:" << list.size();
     }
     
     qsrand(QDateTime::currentDateTime().toTime_t());
     
-    m_obligQuestNr = qMax(list.size() * 4, 20);
+		if (m_level->canBeMelody())
+			m_obligQuestNr = qBound(5, 250 / m_level->melodyLen, 30); // longer melody - less questions
+		else
+			m_obligQuestNr = qBound(20, list.size() * 4, 250);
     if (m_level->useKeySign && !m_level->isSingleKey)
         m_obligQuestNr = qMax(m_obligQuestNr, (m_level->hiKey.value() - m_level->loKey.value() + 1) * 5);
     m_obligQuestNr = qMax(qaPossibilities() * 4, m_obligQuestNr);
 }
 
 
-bool TexecutorSupply::isNoteInKey(Tnote& n) {
-	if (m_level->isSingleKey) {
-		if(m_level->loKey.inKey(n).note != 0)
-				return true;
-		} else {
-				for (int k = m_level->loKey.value(); k <= m_level->hiKey.value(); k++) {
-					if (TkeySignature::inKey(TkeySignature(k), n).note != 0)
-						return true;
-				}
-		}
-	return false;
-}
-
-
-void TexecutorSupply::addToList(QList< TQAunit::TQAgroup >& list, Tnote& n, TfingerPos& f) {
-		TQAunit::TQAgroup g;
-		g.note = n; 
-		g.pos = f;
-		list << g;
-}
-
-
 Tnote TexecutorSupply::determineAccid(Tnote n) {
     Tnote nA = n;
     bool notFound = true;
@@ -250,7 +279,7 @@ Tnote TexecutorSupply::determineAccid(Tnote n) {
             }
         }
         if (notFound && m_prevAccid != Tnote::e_Flat && m_level->withFlats) {
-          if ((n.note == 3 || n.note == 7) && n.acidental == 0 ) { // increase counter for f and c notes
+          if ((n.note == 3 || n.note == 7) && n.alter == 0 ) { // increase counter for f and c notes
             m_eisCesCntr++;
             if (m_eisCesCntr == 3) { // fes or ces can occur every 3 e or b occurences
               m_eisCesCntr = 0;
@@ -263,7 +292,7 @@ Tnote TexecutorSupply::determineAccid(Tnote n) {
           }
         }
         if (notFound && m_prevAccid != Tnote::e_Sharp && m_level->withSharps) {
-          if ((n.note == 4 || n.note == 1) && n.acidental == 0) { // increase counter for f and c notes
+          if ((n.note == 4 || n.note == 1) && n.alter == 0) { // increase counter for f and c notes
             m_eisCesCntr++;
             if (m_eisCesCntr == 3) { // eis or bis can occur every 3 f or c occurences
               nA = n.showWithSharp();
@@ -274,7 +303,7 @@ Tnote TexecutorSupply::determineAccid(Tnote n) {
           }
         }
     }
-    m_prevAccid = (Tnote::Eacidentals)nA.acidental;
+    m_prevAccid = (Tnote::Ealter)nA.alter;
     return nA;
 }
 
@@ -298,7 +327,7 @@ Tnote TexecutorSupply::forceEnharmAccid(Tnote n) {
             nX = n.showWithDoubleSharp();
         if (nX.note && n != nX) break;
      }
-    m_prevAccid = (Tnote::Eacidentals)acc;
+    m_prevAccid = (Tnote::Ealter)acc;
     if (nX.note)
         return nX;
     else return n;
@@ -315,12 +344,12 @@ Tnote::EnameStyle TexecutorSupply::randomNameStyle(int style) {
     if (m_isSolfege) {
         m_isSolfege = false;
         if (qrand() % 2) { // full name like cis, gisis
-            if (gl->seventhIs_B)
+            if (gl->S->seventhIs_B)
                 return Tnote::e_nederl_Bis;
             else
                 return Tnote::e_deutsch_His;
         } else { // name and sign like c#, gx
-            if (gl->seventhIs_B)
+            if (gl->S->seventhIs_B)
                 return Tnote::e_english_Bb;
             else
                 return Tnote::e_norsk_Hb;
@@ -332,21 +361,13 @@ Tnote::EnameStyle TexecutorSupply::randomNameStyle(int style) {
 }
 
 
-quint8 TexecutorSupply::strNr(quint8 str0to6, bool ordered) {
-	if (ordered)
-		return gl->strOrder((char)str0to6);
-	else
-		return str0to6;
-}
-
-
 void TexecutorSupply::getTheSamePos(TfingerPos& fingerPos, QList<TfingerPos>& posList, bool strCheck, bool order) {
-  int chStr = gl->Gtune()->str(strNr(fingerPos.str() - 1, order) + 1).getChromaticNrOfNote();
+  int chStr = gl->Gtune()->str(strNr(fingerPos.str() - 1, order) + 1).chromatic();
   for (int i = 0; i < gl->Gtune()->stringNr(); i++)
     if (i != strNr(fingerPos.str() - 1, order)) { 
       if (strCheck && !m_level->usedStrings[i])
           continue; // skip unavailable strings when strCheck is true
-      int fret = chStr + fingerPos.fret() - gl->Gtune()->str(strNr(i, order) + 1).getChromaticNrOfNote();
+      int fret = chStr + fingerPos.fret() - gl->Gtune()->str(strNr(i, order) + 1).chromatic();
       if (fret >= m_level->loFret && fret <= m_level->hiFret) {
         posList << TfingerPos(strNr(i, order) + 1, fret);
       }
@@ -404,6 +425,155 @@ void TexecutorSupply::calcQAPossibleCount() {
 }
 
 
+void TexecutorSupply::checkNotes(TQAunit* curQ, Tnote& expectedNote, Tnote& userNote, bool reqOctave, bool reqAccid) {
+	Tnote exN = expectedNote, retN = userNote;
+	if (retN.note) {
+		Tnote nE = exN.showAsNatural();
+		Tnote nR = retN.showAsNatural();
+		if (exN != retN) {
+			if (reqOctave) {
+					if (nE.note == nR.note && nE.alter == nR.alter) {
+							if (nE.octave != nR.octave)
+								curQ->setMistake(TQAunit::e_wrongOctave);
+					} else {
+							curQ->setMistake(TQAunit::e_wrongNote);
+					}
+			}
+			if (!curQ->wrongNote()) { // There is still something to check
+				if (exN.note != retN.note || exN.alter != retN.alter) {// if they are equal it means that only octaves were wrong
+						exN = exN.showAsNatural();
+						retN = retN.showAsNatural();
+						if (reqAccid) {
+								if (exN.note == retN.note && exN.alter == retN.alter)
+										curQ->setMistake(TQAunit::e_wrongAccid);
+								else
+										curQ->setMistake(TQAunit::e_wrongNote);
+						} else {
+								if (exN.note != retN.note || exN.alter != retN.alter)
+									curQ->setMistake(TQAunit::e_wrongNote);
+						}
+				}
+			}
+		}
+	} else
+			curQ->setMistake(TQAunit::e_wrongNote);
+}
+
+
+void TexecutorSupply::compareMelodies(Tmelody* q, Tmelody* a, Tattempt* att) {
+	int notesCount = qMax(q->length(), a->length());
+	for (int i = 0; i < notesCount; ++i) {
+		TQAunit tmpUnit;
+		if (a->length() > i && q->length() > i)
+			checkNotes(&tmpUnit, q->note(i)->p(), a->note(i)->p(), m_level->requireOctave, m_level->forceAccids);
+		else
+			tmpUnit.setMistake(TQAunit::e_wrongNote);
+		att->add(tmpUnit.mistake()); // times are ignored in that type of answer/attempt
+	}
+	att->updateEffectiveness();
+}
+
+
+void TexecutorSupply::compareMelodies(Tmelody* q, QList<TnoteStruct>& a, Tattempt* att) {
+// 	debugMelody(q);
+// 	debugNotesStruct(a);
+	int notesCount = qMax(q->length(), a.size());
+	for (int i = 0; i < notesCount; ++i) {
+		TQAunit tmpUnit;
+		if (a.size() > i && q->length() > i) {
+			checkNotes(&tmpUnit, q->note(i)->p(), a[i].pitch, m_level->requireOctave, m_level->forceAccids);
+			if (!tmpUnit.isWrong() && m_level->intonation != TintonationView::e_noCheck) {
+				if (!TnoteStruct(Tnote(), a[i].pitchF).inTune(TintonationView::getThreshold(m_level->intonation)))
+					tmpUnit.setMistake(TQAunit::e_wrongIntonation);
+			}
+		} else
+				tmpUnit.setMistake(TQAunit::e_wrongNote);
+		att->add(tmpUnit.mistake());
+	}
+	att->updateEffectiveness();
+}
+
+
+TkeySignature TexecutorSupply::getKey(Tnote& note) {
+  Tnote tmpNote = note;
+  TkeySignature key; // C-major by default
+  if (m_level->isSingleKey) { //for single key
+    key = m_level->loKey;
+      if (m_level->onlyCurrKey) {
+          tmpNote = m_level->loKey.inKey(note);
+          if (!tmpNote.isValid())
+            qDebug() << "There is no" << tmpNote.toText() << "in level with single key:" << m_level->loKey.getName() <<
+                  "It should never happened!";
+      }
+  } else { // for many key signatures
+      if (m_randKey)
+        key = TkeySignature(m_randKey->get());
+      else 
+        qDebug() << "NO m_randKey WAS CREATED! DO YOU LIKE CRASHES SO MUCH?"; // TODO clear it when OK
+      if (m_level->onlyCurrKey && !m_level->canBeMelody()) { // if note is in current key only
+          int keyRangeWidth = m_level->hiKey.value() - m_level->loKey.value();
+          int patience = 0; // we are looking for suitable key
+          char keyOff = key.value() - m_level->loKey.value();
+          tmpNote = key.inKey(note);
+          while(tmpNote.note == 0 && patience <= keyRangeWidth) {
+              keyOff++;
+              if (keyOff > keyRangeWidth) 
+                keyOff = 0;
+              key = TkeySignature(m_level->loKey.value() + keyOff);
+              patience++;
+              tmpNote = key.inKey(note);
+              if (patience > keyRangeWidth) {
+                  qDebug() << "Oops! It should never happened. Can not find key signature for" << note.toText();
+                  break;
+              }
+          }
+      }
+  }
+  note = tmpNote;
+return key;
+}
+
+
+void TexecutorSupply::resetKeyRandom() {
+  if (m_randKey)
+    m_randKey->reset();
+}
+
+
+
+//##########################################################################################
+//#######################     PRIVATE        ###############################################
+//##########################################################################################
+
+bool TexecutorSupply::isNoteInKey(Tnote& n) {
+	if (m_level->isSingleKey) {
+		if(m_level->loKey.inKey(n).note != 0)
+				return true;
+		} else {
+				for (int k = m_level->loKey.value(); k <= m_level->hiKey.value(); k++) {
+					if (TkeySignature::inKey(TkeySignature(k), n).note != 0)
+						return true;
+				}
+		}
+	return false;
+}
+
+
+void TexecutorSupply::addToList(QList<TQAgroup>& list, Tnote& n, TfingerPos& f) {
+		TQAgroup g;
+		g.note = n; 
+		g.pos = f;
+		list << g;
+}
+
+
+quint8 TexecutorSupply::strNr(quint8 str0to6, bool ordered) {
+	if (ordered)
+		return gl->strOrder((char)str0to6);
+	else
+		return str0to6;
+}
+
 //##########################################################################################
 //#######################     EVENTS        ################################################
 //##########################################################################################
