@@ -17,13 +17,13 @@
  ***************************************************************************/
 
 #include "tfingerboard.h"
-#include "tglobals.h"
-#include "ttune.h"
-#include <tgraphicstexttip.h>
+#include <tglobals.h>
+#include <music/ttune.h>
+#include <graphics/tgraphicstexttip.h>
 #include <animations/tstrikedoutitem.h>
 #include <animations/tcombinedanim.h>
 #include <tcolor.h>
-#include <QtGui>
+#include <QtWidgets>
 #include <QDebug>
 
 extern Tglobals *gl;
@@ -133,58 +133,59 @@ void TfingerBoard::acceptSettings() {
 }
 
 
-void TfingerBoard::setFinger(Tnote note) {
-    if (note.note) {
-				short noteNr = note.getChromaticNrOfNote();
-        bool doShow = true;
-				bool foundPos = false;
-        for(int i = 0; i < gl->Gtune()->stringNr(); i++) { // looking for pos to show
-            int diff = noteNr - gl->Gtune()->str(gl->strOrder(i) + 1).getChromaticNrOfNote();
-            if (doShow && diff >= 0 && diff <= gl->GfretsNumber) { // found
-								foundPos = true;
+void TfingerBoard::setFinger(const Tnote& note) {
+	Tnote n = note; // TODO delete it when Tnote will get const methods
+	if (note.note) {
+			short noteNr = n.chromatic();
+			bool doShow = true;
+			bool foundPos = false;
+			for(int i = 0; i < gl->Gtune()->stringNr(); i++) { // looking for pos to show
+					int diff = noteNr - gl->Gtune()->str(gl->strOrder(i) + 1).chromatic();
+					if (doShow && diff >= 0 && diff <= gl->GfretsNumber) { // found
+							foundPos = true;
 // 								deleteBeyondTip();
-                if (diff == 0) { // open string
-                    m_fingers[gl->strOrder(i)]->hide();
-                    m_strings[gl->strOrder(i)]->show();
-                } else { // some fret
-                    m_strings[gl->strOrder(i)]->hide();
-                    paintFinger(m_fingers[gl->strOrder(i)], gl->strOrder(i), diff);
-										m_fingerPos.setPos(gl->strOrder(i) + i, diff);
-                    m_fingers[gl->strOrder(i)]->show();
-                }
-                if (!gl->GshowOtherPos) {
-                    doShow = false;
-                }
-            } else { // not found on this string or no need to show
-                m_fingers[gl->strOrder(i)]->hide();
-                m_strings[gl->strOrder(i)]->hide();
-            }
-        }
-        if (foundPos)
-					deleteBeyondTip();
-				else if (!m_beyondTip) {
-						m_beyondTip = new TgraphicsTextTip(QString("<span style=\"font-size: %1px; color: %2;\"><br><b> ").
-														arg(height() / 7).arg(gl->EquestionColor.name()) +
-														tr("This note is beyond the scale of the guitar!") + " </b></span><br>", 
-																							 QColor(palette().text().color().darker().name()));
-						if (!gl->GisRightHanded) {
-							QTransform trans;
-							trans.translate(width() / 2, 0);
-							trans.scale(-1, 1);
-							m_beyondTip->setTransform(trans);
-						}
-						m_scene->addItem(m_beyondTip);
-						m_beyondTip->setZValue(150);
-						m_beyondTip->setPos((m_scene->width() - m_beyondTip->boundingRect().width()) / 2,
-												(m_scene->height() - m_beyondTip->boundingRect().height()) / 2);
-				}
-    } else { // hide highlighted fingers/string if no note
-        for (int i = 0; i < gl->Gtune()->stringNr(); i++) {
-            m_fingers[i]->hide();
-            m_strings[i]->hide();
-        }
-    }
-    m_selNote = note;
+							if (diff == 0) { // open string
+									m_fingers[gl->strOrder(i)]->hide();
+									m_strings[gl->strOrder(i)]->show();
+							} else { // some fret
+									m_strings[gl->strOrder(i)]->hide();
+									paintFinger(m_fingers[gl->strOrder(i)], gl->strOrder(i), diff);
+									m_fingerPos.setPos(gl->strOrder(i) + i, diff);
+									m_fingers[gl->strOrder(i)]->show();
+							}
+							if (!gl->GshowOtherPos) {
+									doShow = false;
+							}
+					} else { // not found on this string or no need to show
+							m_fingers[gl->strOrder(i)]->hide();
+							m_strings[gl->strOrder(i)]->hide();
+					}
+			}
+			if (foundPos)
+				deleteBeyondTip();
+			else if (!m_beyondTip) {
+					m_beyondTip = new TgraphicsTextTip(QString("<span style=\"font-size: %1px; color: %2;\"><br><b> ").
+													arg(height() / 7).arg(gl->EquestionColor.name()) +
+													tr("This note is beyond the scale of the guitar!") + " </b></span><br>", 
+																							QColor(qApp->palette().text().color().darker().name()));
+					if (!gl->GisRightHanded) {
+						QTransform trans;
+						trans.translate(width() / 2, 0);
+						trans.scale(-1, 1);
+						m_beyondTip->setTransform(trans);
+					}
+					m_scene->addItem(m_beyondTip);
+					m_beyondTip->setZValue(150);
+					m_beyondTip->setPos((m_scene->width() - m_beyondTip->boundingRect().width()) / 2,
+											(m_scene->height() - m_beyondTip->boundingRect().height()) / 2);
+			}
+	} else { // hide highlighted fingers/string if no note
+			for (int i = 0; i < gl->Gtune()->stringNr(); i++) {
+					m_fingers[i]->hide();
+					m_strings[i]->hide();
+			}
+	}
+	m_selNote = note;
 }
 
 
@@ -347,7 +348,7 @@ void TfingerBoard::showName(Tnote::EnameStyle st, Tnote& note, const QColor& tex
 				qString = m_questString;
 	} else { // try to take note from an answer
 			if (m_fingerPos.fret() != 39 && m_fingerPos.str() != 7 && m_selNote.note) {
-					m_noteName = new TgraphicsTextTip(m_selNote.toRichText(st));
+					m_noteName = new TgraphicsTextTip(note.isValid() ? note.toRichText(st) : m_selNote.toRichText(st));
 					if (m_fingerPos.fret())
 						qFinger = m_fingers[m_fingerPos.str() - 1];
 					else
@@ -417,7 +418,7 @@ void TfingerBoard::createRangeBox(char loFret, char hiFret) {
 
 
 void TfingerBoard::prepareAnswer() {
-    setAttribute(Qt::WA_TransparentForMouseEvents, false); // unlock guitar for mouse
+//     setAttribute(Qt::WA_TransparentForMouseEvents, false); // unlock guitar for mouse
     if (m_rangeBox1)
         m_rangeBox1->show();
     if (m_rangeBox2)
@@ -497,235 +498,258 @@ QRectF TfingerBoard::fingerRect() const {
 
 void TfingerBoard::setTune() {
 	for (int i = 0; i < gl->Gtune()->stringNr(); i++) {
-		if (gl->Gtune()->str(i + 1.5).getChromaticNrOfNote() > 14) { // highest than cis1
+		if (gl->Gtune()->str(i + 1.5).chromatic() > 14) { // highest than cis1
 			m_strColors[i] = QColor(255, 255, 255, 175); // are nylon
 			m_widthFromPitch[i] = 2; // and thiner
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > 10) { // highest than gis
+		} else if (gl->Gtune()->str(i + 1).chromatic() > 10) { // highest than gis
 				m_strColors[i] = QColor(255, 255, 255, 175); // are nylon
 				m_widthFromPitch[i] = 2.5; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > 4) { // highest than dis
+		} else if (gl->Gtune()->str(i + 1).chromatic() > 4) { // highest than dis
 				m_strColors[i] = QColor(255, 255, 255, 125); // are nylon
 				m_widthFromPitch[i] = 3; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > 0) { // highest than b-1(contra)
+		} else if (gl->Gtune()->str(i + 1).chromatic() > 0) { // highest than b-1(contra)
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 3; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -5) { // highest than g-1(contra) 1-st string of bass
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -5) { // highest than g-1(contra) 1-st string of bass
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 3.5; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -10) { // highest than d-1(contra)
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -10) { // highest than d-1(contra)
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 4; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -15) { // highest than gis-2(subcontra)
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -15) { // highest than gis-2(subcontra)
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 4.5; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -20) { // highest than dis-1(contra)
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -20) { // highest than dis-1(contra)
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 5; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -25) { // highest than
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -25) { // highest than
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 6; // and more thick
-		} else if (gl->Gtune()->str(i + 1).getChromaticNrOfNote() > -30) { // highest than
+		} else if (gl->Gtune()->str(i + 1).chromatic() > -30) { // highest than
 				m_strColors[i] = QColor("#C29432"); // are gold-plated
 				m_widthFromPitch[i] = 7; // and more thick
 		}
 	}
-	m_loNote = gl->loString().getChromaticNrOfNote();
-	m_hiNote = gl->hiString().getChromaticNrOfNote() + gl->GfretsNumber;
+	m_loNote = gl->loString().chromatic();
+	m_hiNote = gl->hiString().chromatic() + gl->GfretsNumber;
 }
 
 
 void TfingerBoard::paint() {
-    m_fbRect = QRect(10, height() / 18, (6 * width()) / 7, height() - height() / 18);
-    m_fretWidth = ((m_fbRect.width() + ((gl->GfretsNumber / 2)*(gl->GfretsNumber / 2 + 1))
-                  + gl->GfretsNumber / 4) / (gl->GfretsNumber+1)) + 1;
+	m_fbRect = QRect(10, height() / 18, (6 * width()) / 7, height() - height() / 18);
+	m_fretWidth = ((m_fbRect.width() + ((gl->GfretsNumber / 2)*(gl->GfretsNumber / 2 + 1))
+								+ gl->GfretsNumber / 4) / (gl->GfretsNumber+1)) + 1;
 //     m_strGap = m_fbRect.height() / 6;
-		m_strGap = m_fbRect.height() / gl->Gtune()->stringNr();
-    m_fretsPos[0] = m_fbRect.x() + m_fretWidth;
-    for (int i = 2; i < gl->GfretsNumber + 1; i++)
-        m_fretsPos[i - 1] = m_fretsPos[i - 2] + (m_fretWidth-(i / 2));
-    lastFret = m_fretsPos[gl->GfretsNumber - 1];
-    if (lastFret > (m_fbRect.width() + 10)) {
-        m_fbRect.setWidth(lastFret - 8);
-        qDebug("fretboard size changed");
-    }
-  // Let's paint
-    QPixmap pixmap(size());
-    pixmap.fill(QColor(palette().window().color()));
+	m_strGap = m_fbRect.height() / gl->Gtune()->stringNr();
+	m_fretsPos[0] = m_fbRect.x() + m_fretWidth;
+	for (int i = 2; i < gl->GfretsNumber + 1; i++)
+			m_fretsPos[i - 1] = m_fretsPos[i - 2] + (m_fretWidth-(i / 2));
+	lastFret = m_fretsPos[gl->GfretsNumber - 1];
+	if (lastFret > (m_fbRect.width() + 10)) {
+			m_fbRect.setWidth(lastFret - 8);
+			qDebug("fretboard size changed");
+	}
+// Let's paint
+	QPixmap pixmap(size());
+	pixmap.fill(QColor(palette().window().color()));
 
-    QPainter painter(&pixmap);
-    painter.setRenderHint(QPainter::Antialiasing, true);
-    painter.setRenderHint(QPainter::TextAntialiasing, true);
-    painter.setWindow(0 , 0, width(), height());
-    resetTransform();
-    if (!gl->GisRightHanded) {
-        translate(width(), 0);
-        scale(-1, 1);
-    }
-  // FINGERBOARD
-    painter.setPen(QPen(Qt::black, 0, Qt::NoPen));
-		if (gl->instrument == e_classicalGuitar)
-				painter.setBrush(QBrush(Qt::black, Qt::SolidPattern));
-		else {
-				QColor fbEdgeColor = QColor("#AFA072");
-				fbEdgeColor.setAlpha(220);
-				painter.setBrush(QBrush(fbEdgeColor, Qt::SolidPattern));
+	QPainter painter(&pixmap);
+	painter.setRenderHint(QPainter::Antialiasing, true);
+	painter.setRenderHint(QPainter::TextAntialiasing, true);
+	painter.setWindow(0 , 0, width(), height());
+	resetTransform();
+	if (!gl->GisRightHanded) {
+			translate(width(), 0);
+			scale(-1, 1);
+	}
+// FINGERBOARD
+	painter.setPen(QPen(Qt::black, 0, Qt::NoPen));
+	if (gl->instrument == e_classicalGuitar)
+			painter.setBrush(QBrush(Qt::black, Qt::SolidPattern));
+	else {
+			QColor fbEdgeColor = QColor("#AFA072");
+			fbEdgeColor.setAlpha(220);
+			painter.setBrush(QBrush(fbEdgeColor, Qt::SolidPattern));
+	}
+	QPolygon a;
+	int fbThick = ((m_strGap * gl->Gtune()->stringNr()) / 6) / 4; // thickness of fretboard
+	a.setPoints(6, m_fbRect.x() - 6, m_fbRect.y(),
+							m_fbRect.x() + 2, m_fbRect.y() - fbThick,
+							m_fbRect.x() + m_fbRect.width() + fbThick, m_fbRect.y() - fbThick,
+							m_fbRect.x() + m_fbRect.width() + fbThick, height() - fbThick,
+							m_fbRect.x() + m_fbRect.width(), height(),
+							m_fbRect.x(), height());
+	painter.drawPolygon(a);
+	if (gl->instrument == e_classicalGuitar)
+			painter.setBrush(QBrush(QPixmap(gl->path + "picts/fingbg.png")));
+	else
+			painter.setBrush(QBrush(QPixmap(gl->path + "picts/fingbg-el.png")));
+	painter.drawRect(m_fbRect);
+// FRETS
+	// zero fret (upper bridge or HUESO)
+	if (gl->instrument == e_classicalGuitar) {
+			painter.setPen(Qt::NoPen);
+			painter.setBrush(QBrush(QColor("#FFFBF0"),Qt::SolidPattern)); //#FFFBF0 cream color for hueso
+			painter.drawRect(m_fbRect.x() - 8, m_fbRect.y() + 4, 7, m_fbRect.height());
+			a.setPoints(4, m_fbRect.x() - 8,							m_fbRect.y() + 4,
+									m_fbRect.x() - 1, 								m_fbRect.y() + 4,
+									m_fbRect.x() + m_strGap / 3 - 1,	m_fbRect.y() - m_strGap / 3,
+									m_fbRect.x() + m_strGap / 3 - 8,	m_fbRect.y() - m_strGap / 3);
+			painter.setBrush(QBrush(QColor("#FFEEBC"),Qt::SolidPattern)); // a bit darker for its rant
+			painter.drawPolygon(a);
+	} else {
+			QLinearGradient fretGrad(fbRect().x() - 8, 10.0, fbRect().x() + 6, 10.0);
+			fretGrad.setColorAt(0.0, QColor("#DAE4E4"));
+			fretGrad.setColorAt(0.4, QColor("#7F806E"));
+			fretGrad.setColorAt(0.7, QColor("#3B382B"));
+			fretGrad.setColorAt(0.9, QColor("#000000"));
+			painter.setBrush(fretGrad);
+			painter.drawRoundedRect(fbRect().x() - 8, m_fbRect.y() + 2, 14, m_fbRect.height() - 4, 2, 2);
+	}
+// others frets
+	qint8 fretMarks[gl->GfretsNumber]; // array keeps whether fret is marked with dots (1) or two (2)
+	for (int i = 0; i < gl->GfretsNumber; ++i)
+		fretMarks[i] = 0;
+	for (int fr = 0; fr < gl->GmarkedFrets.size(); ++fr) {
+		QString exMark = "", frTxt = gl->GmarkedFrets[fr].toString();
+		if (frTxt.contains("!")) {
+			exMark = "!";
+			frTxt.replace("!", "");
 		}
-    QPolygon a;
-		int fbThick = ((m_strGap * gl->Gtune()->stringNr()) / 6) / 4; // thickness of fretboard
-    a.setPoints(6, m_fbRect.x() - 6, m_fbRect.y(),
-								m_fbRect.x() + 2, m_fbRect.y() - fbThick,
-                m_fbRect.x() + m_fbRect.width() + fbThick, m_fbRect.y() - fbThick,
-                m_fbRect.x() + m_fbRect.width() + fbThick, height() - fbThick,
-                m_fbRect.x() + m_fbRect.width(), height(),
-                m_fbRect.x(), height());
-    painter.drawPolygon(a);
-		if (gl->instrument == e_classicalGuitar)
-				painter.setBrush(QBrush(QPixmap(gl->path + "picts/fingbg.png")));
-		else
-				painter.setBrush(QBrush(QPixmap(gl->path + "picts/fingbg-el.png")));
-    painter.drawRect(m_fbRect);
-  // FRETS
-   // zero fret (upper bridge or HUESO)
-		if (gl->instrument == e_classicalGuitar) {
-				painter.setPen(Qt::NoPen);
-				painter.setBrush(QBrush(QColor("#FFFBF0"),Qt::SolidPattern)); //#FFFBF0 cream color for hueso
-				painter.drawRect(m_fbRect.x() - 8, m_fbRect.y() + 4, 7, m_fbRect.height());
-				a.setPoints(4, m_fbRect.x() - 8,							m_fbRect.y() + 4,
-										m_fbRect.x() - 1, 								m_fbRect.y() + 4,
-										m_fbRect.x() + m_strGap / 3 - 1,	m_fbRect.y() - m_strGap / 3,
-										m_fbRect.x() + m_strGap / 3 - 8,	m_fbRect.y() - m_strGap / 3);
-				painter.setBrush(QBrush(QColor("#FFEEBC"),Qt::SolidPattern)); // a bit darker for its rant
-				painter.drawPolygon(a);
-		} else {
-				QLinearGradient fretGrad(fbRect().x() - 8, 10.0, fbRect().x() + 6, 10.0);
-        fretGrad.setColorAt(0.0, QColor("#DAE4E4"));
-        fretGrad.setColorAt(0.4, QColor("#7F806E"));
-        fretGrad.setColorAt(0.7, QColor("#3B382B"));
-        fretGrad.setColorAt(0.9, QColor("#000000"));
-        painter.setBrush(fretGrad);
-        painter.drawRoundedRect(fbRect().x() - 8, m_fbRect.y() + 2, 14, m_fbRect.height() - 4, 2, 2);
-		}
- // others frets
-    for (int i = 0; i < gl->GfretsNumber; i++) {
-        QLinearGradient fretGrad(m_fretsPos[i], 10.0, m_fretsPos[i] + 8, 10.0);
-        fretGrad.setColorAt(0.0, QColor("#DAE4E4"));
-        fretGrad.setColorAt(0.4, QColor("#7F806E"));
-        fretGrad.setColorAt(0.7, QColor("#3B382B"));
-        fretGrad.setColorAt(0.9, QColor("#000000"));
-        painter.setBrush(fretGrad);
-        painter.drawRoundedRect(m_fretsPos[i], m_fbRect.y() + 2, 9, m_fbRect.height() - 4, 2, 2);
-        if ( (gl->instrument == e_bassGuitar && i == 2) || i==4 || i==6 || i==8 || i==14 || i==16)	{
-                //white color for circles marking 5,7,9... frets
-            painter.setBrush(QBrush(Qt::white, Qt::SolidPattern));
-            painter.drawEllipse(m_fretsPos[i] - 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
-                                m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
-        } else if (i==11)	{
-            painter.setBrush(QBrush(Qt::white, Qt::SolidPattern));
-            painter.drawEllipse(m_fretsPos[i] - 12 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
-                                m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
-						painter.drawEllipse(m_fretsPos[i] + 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
-                                m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
-        }
-    }
-  // STRINGS
-    QFont strFont = font();
-    strFont.setPixelSize((int)qRound(0.75 * m_strGap));//setting font for digits
-    painter.setFont(strFont);
-    painter.setBrush(QBrush(Qt::NoBrush));
-    for (int i = 0; i < 6; i++) {
-//  drawing main strings
-			qreal lineYpos = m_fbRect.y() + m_strGap / 2 + i * m_strGap;
-			QLinearGradient strGrad(1.0, lineYpos - m_strWidth[i] / 2, 1.0, lineYpos + m_strWidth[i] / 2);
-			strGrad.setColorAt(0.0, m_strColors[i]);
-			strGrad.setColorAt(0.5, m_strColors[i].darker());
-			painter.setPen(QPen(QBrush(strGrad), m_strWidth[i], Qt::SolidLine));
-			painter.drawLine(1, lineYpos, width() - 1 - m_strGap, lineYpos);
-			if (i < gl->Gtune()->stringNr()) {
-					m_workStrings[i]->setPen(QPen(gl->GfingerColor, m_strWidth[i] + 2, Qt::SolidLine));
-					m_workStrings[i]->setLine(1, lineYpos, width() - 1 - m_strGap, lineYpos);
-					m_strings[i]->setPen(QPen(gl->GselectedColor, m_strWidth[i], Qt::SolidLine));
-					m_strings[i]->setLine(m_workStrings[i]->line());
-					m_strings[i]->show();
-// drawing digits of strings in circles
-					painter.setPen(QPen(m_strColors[i], 1, Qt::SolidLine));
-					painter.setBrush(QBrush(QColor(0, 0, 0, 180)));
-					int circleX;
-					if (!gl->GisRightHanded) {
-							painter.scale (-1, 1);
-							painter.translate(-width(), 0);
-							circleX = 1;
-					} else
-							circleX = width() - 1 - m_strGap;
-					painter.drawEllipse(circleX, m_fbRect.y() + i * m_strGap, m_strGap - 1, m_strGap - 1);
-					painter.setPen(QPen(Qt::green,1,Qt::SolidLine));// in green color
-					if (gl->GisRightHanded)
-							painter.drawText(width() - ((int)qreal(m_strGap * 0.75)),
-															m_fbRect.y() + (int)qreal(m_strGap * (0.75 + i)), QString::number(i + 1));
-						else {
-								painter.drawText((int)qreal(m_strGap * 0.28),
-																m_fbRect.y()+(int)qreal(m_strGap * (0.75 + i)), QString::number(i + 1));
-								painter.translate(width(), 0);
-								painter.scale (-1, 1);
-						}
-		// shadow of the strings
-						painter.setPen(QPen(QColor(0, 0, 0, 150), m_strWidth[i], Qt::SolidLine)); // on the fingerboard
-// 							int yy = m_fbRect.y() + m_strGap / 2 + (i - 1) * m_strGap - 3 - m_strWidth[i];
-						int yy = lineYpos + m_strGap * 0.2;
-						painter.drawLine(m_fbRect.x() + 3, yy, m_fbRect.x() + m_fbRect.width() - 1, yy);
-						painter.setPen(QPen(Qt::black, 1, Qt::SolidLine)); //on upper bridge
-						painter.drawLine(m_fbRect.x() - 8, lineYpos - 2,
-														m_fbRect.x(), lineYpos - 2);
-						painter.drawLine(m_fbRect.x() - 8, lineYpos + m_strWidth[i] - 1,
-														m_fbRect.x() - 1, lineYpos + m_strWidth[i] - 1);
-						if (m_pickRect->width()) { // shadow on the pickup if exist (bass or electric guitar)
-							int pickX = m_pickRect->x();
-							if (!gl->GisRightHanded)
-									pickX = width() - (m_pickRect->x() + m_pickRect->width());
-							painter.setPen(QPen(QColor(28, 28, 28, 30), m_strWidth[i], Qt::SolidLine));
-							yy += m_strGap * 0.1;
-							int subW = qRound((qreal)m_pickRect->width() * 0.15);
-							QPoint ps[5] = {
-										QPoint(m_fbRect.x() + m_fbRect.width() + fbThick, yy + m_strGap * 0.12),
-										QPoint(pickX, yy + m_strGap * 0.12),
-										QPoint(pickX + subW, yy),
-										QPoint(pickX + m_pickRect->width() - subW, yy),
-										QPoint(pickX + m_pickRect->width() - subW * 0.3, yy + m_strGap * 0.15)
-							};
-							painter.drawPolyline(ps, 5);
-						}							
-
-			} else { // hide rest lines (strings)
-					m_workStrings[i]->hide();
-					m_strings[i]->hide();
-			}
-		}
-    m_workFinger->setRect(0, 0, m_fretWidth / 1.6, qRound(0.7 * m_strGap));
-    for (int i = 0; i < 6; i++) {
-			if (i < gl->Gtune()->stringNr()) {
-        m_fingers[i]->setRect(m_workFinger->rect());
-				m_fingers[i]->show();
-			}
+		bool ok;
+		int frNr = frTxt.toInt(&ok);
+		if (ok && frNr > 0 && frNr <= gl->GfretsNumber) {
+			if (exMark.isEmpty())
+				fretMarks[frNr - 1] = 1;
 			else
-				m_fingers[i]->hide();
+				fretMarks[frNr - 1] = 2;
 		}
+	}
+	for (int i = 0; i < gl->GfretsNumber; i++) {
+			QLinearGradient fretGrad(m_fretsPos[i], 10.0, m_fretsPos[i] + 8, 10.0);
+			fretGrad.setColorAt(0.0, QColor("#DAE4E4"));
+			fretGrad.setColorAt(0.4, QColor("#7F806E"));
+			fretGrad.setColorAt(0.7, QColor("#3B382B"));
+			fretGrad.setColorAt(0.9, QColor("#000000"));
+			painter.setBrush(fretGrad);
+			painter.drawRoundedRect(m_fretsPos[i], m_fbRect.y() + 2, 9, m_fbRect.height() - 4, 2, 2);
+			
+			if (fretMarks[i])	{
+				painter.setBrush(QBrush(Qt::white, Qt::SolidPattern)); //white color for circles marking 5, 7, 9... frets
+				if (fretMarks[i] == 1)
+					painter.drawEllipse(m_fretsPos[i] - 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
+															m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
+				else {
+// 					painter.drawEllipse(m_fretsPos[i] - 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
+// 															m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 10, 8, 8);
+// 					painter.drawEllipse(m_fretsPos[i] - 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
+// 															m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) + 8, 8, 8);
+					painter.drawEllipse(m_fretsPos[i] - 12 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
+															m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
+					painter.drawEllipse(m_fretsPos[i] + 4 - (m_fretsPos[i] - m_fretsPos[i - 1]) / 2,
+															m_fbRect.y() + m_strGap * (int)(gl->Gtune()->stringNr() / 2) - 2, 8, 8);
+				}
+			}
+	}
+// STRINGS
+	QFont strFont = font();
+	strFont.setPixelSize((int)qRound(0.75 * m_strGap));//setting font for digits
+	painter.setFont(strFont);
+	painter.setBrush(QBrush(Qt::NoBrush));
+	for (int i = 0; i < 6; i++) {
+//  drawing main strings
+		qreal lineYpos = m_fbRect.y() + m_strGap / 2 + i * m_strGap;
+		QLinearGradient strGrad(1.0, lineYpos - m_strWidth[i] / 2, 1.0, lineYpos + m_strWidth[i] / 2);
+		strGrad.setColorAt(0.0, m_strColors[i]);
+		strGrad.setColorAt(0.5, m_strColors[i].darker());
+		painter.setPen(QPen(QBrush(strGrad), m_strWidth[i], Qt::SolidLine));
+		painter.drawLine(1, lineYpos, width() - 1 - m_strGap, lineYpos);
+		if (i < gl->Gtune()->stringNr()) {
+				m_workStrings[i]->setPen(QPen(gl->GfingerColor, m_strWidth[i] + 2, Qt::SolidLine));
+				m_workStrings[i]->setLine(1, lineYpos, width() - 1 - m_strGap, lineYpos);
+				m_strings[i]->setPen(QPen(gl->GselectedColor, m_strWidth[i], Qt::SolidLine));
+				m_strings[i]->setLine(m_workStrings[i]->line());
+				m_strings[i]->show();
+// drawing digits of strings in circles
+				painter.setPen(QPen(m_strColors[i], 1, Qt::SolidLine));
+				painter.setBrush(QBrush(QColor(0, 0, 0, 180)));
+				int circleX;
+				if (!gl->GisRightHanded) {
+						painter.scale (-1, 1);
+						painter.translate(-width(), 0);
+						circleX = 1;
+				} else
+						circleX = width() - 1 - m_strGap;
+				painter.drawEllipse(circleX, m_fbRect.y() + i * m_strGap, m_strGap - 1, m_strGap - 1);
+				painter.setPen(QPen(Qt::green,1,Qt::SolidLine));// in green color
+				if (gl->GisRightHanded)
+						painter.drawText(width() - ((int)qreal(m_strGap * 0.75)),
+														m_fbRect.y() + (int)qreal(m_strGap * (0.75 + i)), QString::number(i + 1));
+					else {
+							painter.drawText((int)qreal(m_strGap * 0.28),
+															m_fbRect.y()+(int)qreal(m_strGap * (0.75 + i)), QString::number(i + 1));
+							painter.translate(width(), 0);
+							painter.scale (-1, 1);
+					}
+	// shadow of the strings
+					painter.setPen(QPen(QColor(0, 0, 0, 150), m_strWidth[i], Qt::SolidLine)); // on the fingerboard
+// 							int yy = m_fbRect.y() + m_strGap / 2 + (i - 1) * m_strGap - 3 - m_strWidth[i];
+					int yy = lineYpos + m_strGap * 0.2;
+					painter.drawLine(m_fbRect.x() + 3, yy, m_fbRect.x() + m_fbRect.width() - 1, yy);
+					painter.setPen(QPen(Qt::black, 1, Qt::SolidLine)); //on upper bridge
+					painter.drawLine(m_fbRect.x() - 8, lineYpos - 2,
+													m_fbRect.x(), lineYpos - 2);
+					painter.drawLine(m_fbRect.x() - 8, lineYpos + m_strWidth[i] - 1,
+													m_fbRect.x() - 1, lineYpos + m_strWidth[i] - 1);
+					if (m_pickRect->width()) { // shadow on the pickup if exist (bass or electric guitar)
+						int pickX = m_pickRect->x();
+						if (!gl->GisRightHanded)
+								pickX = width() - (m_pickRect->x() + m_pickRect->width());
+						painter.setPen(QPen(QColor(28, 28, 28, 30), m_strWidth[i], Qt::SolidLine));
+						yy += m_strGap * 0.1;
+						int subW = qRound((qreal)m_pickRect->width() * 0.15);
+						QPoint ps[5] = {
+									QPoint(m_fbRect.x() + m_fbRect.width() + fbThick, yy + m_strGap * 0.12),
+									QPoint(pickX, yy + m_strGap * 0.12),
+									QPoint(pickX + subW, yy),
+									QPoint(pickX + m_pickRect->width() - subW, yy),
+									QPoint(pickX + m_pickRect->width() - subW * 0.3, yy + m_strGap * 0.15)
+						};
+						painter.drawPolyline(ps, 5);
+					}							
 
-    if (m_questFinger) {
-        m_questFinger->setRect(m_workFinger->rect());
-        paintFinger(m_questFinger, m_questPos.str() - 1, m_questPos.fret());
-    }
-    if (m_questString)
-        m_questString->setLine(m_strings[m_questPos.str() - 1]->line());
-    if (m_questMark) paintQuestMark();
-    resizeRangeBox();
-    m_scene->setBackgroundBrush(QBrush(pixmap));
-    setFinger(m_selNote);
-    if (m_highString)
-      setHighlitedString(m_hilightedStrNr);
+		} else { // hide rest lines (strings)
+				m_workStrings[i]->hide();
+				m_strings[i]->hide();
+		}
+	}
+	m_workFinger->setRect(0, 0, m_fretWidth / 1.6, qRound(0.7 * m_strGap));
+	for (int i = 0; i < 6; i++) {
+		if (i < gl->Gtune()->stringNr()) {
+			m_fingers[i]->setRect(m_workFinger->rect());
+			m_fingers[i]->show();
+		}
+		else
+			m_fingers[i]->hide();
+	}
+
+	if (m_questFinger) {
+			m_questFinger->setRect(m_workFinger->rect());
+			paintFinger(m_questFinger, m_questPos.str() - 1, m_questPos.fret());
+	}
+	if (m_questString)
+			m_questString->setLine(m_strings[m_questPos.str() - 1]->line());
+	if (m_questMark) paintQuestMark();
+	resizeRangeBox();
+	m_scene->setBackgroundBrush(QBrush(pixmap));
+	setFinger(m_selNote);
+	if (m_highString)
+		setHighlitedString(m_hilightedStrNr);
 }
 
 
 Tnote TfingerBoard::posToNote(int str, int fret) {
-    return Tnote(gl->Gtune()->str(str + 1).getChromaticNrOfNote() + fret);
+    return Tnote(gl->Gtune()->str(str + 1).chromatic() + fret);
 }
 
 
@@ -774,9 +798,6 @@ void TfingerBoard::mousePressEvent(QMouseEvent *event) {
 		if (m_curFret != 99 && m_curStr != 7) {
 			m_selNote = posToNote(m_curStr, m_curFret);
 			m_fingerPos = TfingerPos(m_curStr + 1, m_curFret);
-			if (gl->GpreferFlats)
-				if (m_selNote.note != 3 && m_selNote.note != 7) // eliminate Fb from E and Cb from B
-					m_selNote = m_selNote.showWithFlat();
 			if (gl->GshowOtherPos)
 				setFinger(m_selNote);
 			else
@@ -784,7 +805,6 @@ void TfingerBoard::mousePressEvent(QMouseEvent *event) {
 			emit guitarClicked(m_selNote);
 		} else {
 			m_selNote = Tnote(0,0,0);
-	//        setFinger(m_selNote);
 		}
 	}
 }
@@ -816,7 +836,9 @@ void TfingerBoard::paintQuestMark() {
     m_questMark->setFont(f);
     int off = -1, off2 = 0;
     if (!gl->GisRightHanded) {
-        m_questMark->scale(-1, 1);
+				QTransform matrix;
+				matrix.scale(-1, 1);
+				m_questMark->setTransform(matrix, true);
         if (m_questPos.fret() == 1)
             off = 1;
         else
@@ -997,16 +1019,17 @@ void TfingerBoard::strikeBlinkingFinished() {
 
 
 void TfingerBoard::finishCorrection() {
-		m_animation->deleteLater();
-		m_animation = 0;
-		if (m_movingItem) {
-			delete m_movingItem;
-			m_movingItem = 0;
-		}
-		setFinger(m_goodPos);
-		markAnswer(QColor(gl->EanswerColor.lighter().name()));
-		if (m_nameInCorrection)
-				showName(m_corrStyle, gl->EanswerColor);
+  m_animation->deleteLater();
+  m_animation = 0;
+  if (m_movingItem) {
+    delete m_movingItem;
+    m_movingItem = 0;
+  }
+  setFinger(m_goodPos);
+  markAnswer(QColor(gl->EanswerColor.lighter().name()));
+  if (m_nameInCorrection)
+      showName(m_corrStyle, gl->EanswerColor);
+  emit correctingFinished();
 }
 
 
